*	PC260QA.ASM = Device Driver for PC mounted H8D images
*		for use with h89mount.cpp developed with Qt 4.5
*	Modified 2/11/2013 from PCDVD6.ASM:
*		Changed from port 340Q/341Q to port 260Q/262Q
*		Changed sense of FT245 status signals
*		Further modified to complete conversion of status sigs
*
*	Comment out debuging statements that display the
*		Command Packet.
*
*	Add mount unit number to Command Packet
*
*	Now working.  Remove all debug statements - 2/28/2010, 3:00 PM
*
*	Add dump of last 5 bytes of each sector read
*
*
	XTEXT	HOSEQU
	XTEXT	ECDEF

	XTEXT	PICDEF
	XTEXT	H17DEF
	XTEXT	DIRDEF

	XTEXT	ASCII
	XTEXT	DEVDEF
	XTEXT	DVDDEF
	XTEXT	ESINT
	XTEXT	EDRAM
	XTEXT	SETCAL

**	Assembly Constants
*

SYMNU	EQU	4		maximum number of units
PCENT	EQU	002000A 	dvd entry point address
$DADA	EQU	030072A
$TBRA	EQU	031076A
$TYPTX	EQU	031136A
$UDD	EQU	031157A

DC.MAX	EQU	013Q

*	FDTI port definitions


USBDAT	EQU	260Q
USBSTAT EQU	262Q
USBRX	EQU	2
USBTX	EQU	1

	CODE	PIC

SYCAP	EQU	DT.CW+DT.CR+DT.DD+DT.RN 	Read, Write, Directory, Random

	DB	DVDFLV			DEVICE DRIVER FLAG
	DB	SYCAP			Device Capabilities
	DB	0			Mounted Units Mask
	DB	SYMNU			Maximum of Number of Units
	DB	SYCAP			0:  Read, Write, Directory, Random
	DB	SYCAP			1:  Read, Write, Directory, Random
	DB	SYCAP			2:  Read, Write, Diredtory, Random
	DB	SYCAP			3:  Read, Write, Directory, Random
	DB	0			4:  Ignored
	DB	0			5:  Ignored
	DB	0			6:  Ignored
	DB	0			7:  Ignored
	DB	DVDFLV			Device driver flag

.	SET	023Q
	ERRNZ	*-.
	DS	DVD.STE-.

***	Set Dispatch
*	Set Commands enter here
*
*	ENTRY:	A	= unit number
*		DE	= line pointer
*
*	EXIT:	PSW	= 'C' clear if NO error
*			  'C' set   if	  error
*
*	USES:	ALL
*

SETNTR	EQU	*
	ERRNZ	*-DVD.STE

	MOV	B,D
	MOV	C,E
	LXI	D,PRCTAB	DE = processor table
	LXI	H,OPTTAB	HL = option table
	CALL	$SOP
	RC			ERROR

	CALL	$SNA
	RZ			At the end of the line

	MVI	A,EC.ILO
	STC			Other trailing junk
	RET

***	Processors
**	HELP	- Process HELP Options

HELP	CALL	$TYPTX
	DB	NL,NL,'Set Options:',NL,NL
	DB	'HELP',TAB,TAB,'Type this message',NL
	DB	TAB,TAB,'There are no other set options',NL
	DB	NL,ENL
	XRA	A
	RET

OPTTAB	DW	OPTABE		End of Option Table
	DB	6		6 data bytes

	DB	'HEL','P'+200Q,HELPI
	DB	0,0,0,0,0

OPTABE	DB	0


**	Processor Table
*

PRCTAB	DS	0

HELPI	EQU	*-PRCTAB/2
	DW	HELP

.	SET	237A
	ERRNZ	*-.		check value of SET
	ERRMI	*-.		show if minus value

	DS	DVD.ENT-.,0

***	Driver Dispatch
*
*	ENTRY:	A	=  Driver function code
*	EXIT:	???
*	USES:	ALL
*
PCDVD	EQU	*
	ERRNZ	*-DVD.ENT

ENTRY	CPI	DC.MAX
	JC	PC1		Legal entry point

	MVI	A,EC.ILR	Illegal Driver Call
	STC
	RET

*	Dispatch Valid Driver Call

PC1	CALL	$TBRA
PC1.	DB	PCREAD-*	Read
	DB	PCWRITE-*	Write
	DB	PCREADR-*	Read Regardless
	DB	PCOPNR-*	Open for read
	DB	PCOPNW-*	Open for write
	DB	PCOPNU-*	Open for update
	DB	PCCLOS-*	Close
	DB	PCABRT-*	Abort
	DB	PCMOUN-*	Mount device
	DB	PCLOAD-*	Load
	DB	PCREDY-*	Ready
ENT.CD	EQU	*-PC1.
	ERRNZ	ENT.CD-DC.MAX		be sure 11 codes handled

*	PCLOAD process the device LOAD entry.

PCLOAD	EQU	*
	ANA	A	Nothing to do upon load . . .
	RET

PCWRITE JMP	PCWRI.

PCREAD	EQU	*
PCREADR JMP	PCREAD. 	read regardless same as read

PCREDY	JMP	PCREDY. 	"drive" always ready
PCABRT	JMP	PCABRT. 	Abort = nop
PCMOUN	JMP	PCMOUN. 	Mount, no action req'd

PCOPNR	EQU	*		Open = nop
PCOPNW	EQU	*		Open = nop
PCOPNU	EQU	*		Open = nop
PCCLOS	EQU	*		Close = nop
PCNOP	EQU	*		nop
PCREDY. EQU	*
PCABRT. EQU	*
PCMOUN. EQU	*
	ANA	A
	RET


*	WRITE TO PC mounted H8D image
*
*	(BC)   = COUNT
*	(DE)   = ADDRESS OF SOURCE DATA
*	(HL)   = SECTOR NUMBER

PCWRI.	EQU	*

* Alert PC that we want to write (command packet)
* Get confirmation that PC is ready (image loaded, port open, etc.)
* Send data packet with sector data to write.
* Get confirmation that PC write was OK.
* Return

*  If called w/BC = 0, dummy write testing for write protect

	MOV	A,B
	ORA	C
	RZ

	MOV	A,B		save # sectors to write
	STA	LOOPCNT

	LDA	AIO.UNI
	STA	WRI.UNI

	SHLD	SEC.NUM
	XCHG
	SHLD	SRC.DAT
	XCHG

SND.SEC EQU	*
	MOV	A,H
	STA	WRI.SEC+1	put sector number into command packet
	MOV	A,L
	STA	WRI.SEC

	MVI	B,12		count of bytes to send
	LXI	H,WRI.BYT	Command Packet to send
	CALL	SEND.IT

	LXI	H,ACK.BUF
	CALL	RED.PKT 	check for valid acknowledgement
	CALL	CHK.ACK
	RC			return with error code in ACC

	MVI	B,2
	LXI	H,SEC.BYT
	CALL	SEND.IT

	MVI	B,0		count of bytes to send 0=>256
	LHLD	SRC.DAT 	pointer to source data
	CALL	SEND.IT

	MVI	B,1		count of bytes to send
	LXI	H,SEC.CHK
	CALL	SEND.IT

	LXI	H,ACK.BUF
	CALL	RED.PKT 	check for valid acknowledgement
	CALL	CHK.ACK
	RC
*	RET			force return for debug

	LDA	LOOPCNT
	DCR	A
	STA	LOOPCNT
	RZ

	LHLD	SRC.DAT
	INR	H
	SHLD	SRC.DAT 	prepare next 256 bytes from source

	LHLD	SEC.NUM 	more to go, so inc. sector number
	INX	H
	SHLD	SEC.NUM 	re-save it
	JNZ	SND.SEC
	RET			this far w/o error, we're done


WRI.BYT DB	12,0		byte count
WRI.CMD DB	2		command to write
WRI.SEC DB	0,0		sector number
WRI.CNT DB	0,1		data bytes to send (256 default)
WRI.UNI DB	0		unit number
	DB	0,0,0		reserved
WRI.CHK DB	0		command packet checksum

SEC.BYT DB	3,1		byte count to send (decimal 259)
SEC.CHK DB	0		data packet checksum


*	READ FROM PC Mounted H8D image
*
*	(BC) = count
*	(DE) = address of destination of data
*	(HL) = sector #
*

PCREAD. EQU	*
* Alert PC that we want to read a sector (command packet)
* Get confirmation that PC is ready (image loaded, port open, etc.)
* PC will pause briefly and then start sending data
* Return

	LDA	AIO.UNI
	STA	RED.UNI

	SHLD	SEC.NUM
	XCHG
	SHLD	DST.DAT 	destination for data
	XCHG

	MOV	A,H
	STA	RED.SEC+1	put sector number into command packet
	MOV	A,L
	STA	RED.SEC

	MOV	A,B
	STA	RED.CNT+1
	MOV	A,C
	STA	RED.CNT 	number of bytes to read

	MVI	B,12		count of bytes to send
	MVI	C,USBDAT	port destination
	LXI	H,RED.BYT	data to send
	CALL	SEND.IT

	LXI	H,ACK.BUF
	CALL	RED.PKT 	check for valid acknowledgement
	CALL	CHK.ACK
	RC			return with error code in ACC

	LHLD	DST.DAT
	CALL	RED.PKT
*  return with (DE) = next unused address
*  and with (BC) = unused byte count (0)

	XRA	A
	RET

RED.BYT DB	12,0		byte count
RED.CMD DB	3		command to write
RED.SEC DB	0,0		sector number
RED.CNT DB	0,0		bytes to read from image
RED.UNI DB	0		unit number
	DB	0,0,0		reserved
RED.CHK DB	0		command packet checksum

SEND.IT EQU	*		send 256 bytes or less
**	Enter with HL pointing to data,
**	(B)= byte count, 0=>256
**

SEND.1	IN	USBSTAT
	ANI	USBTX
	JZ	SEND.1
*	JNZ	SEND.1

	MOV	A,M
	OUT	USBDAT
	INX	H
	DCR	B
	JNZ	SEND.1
	RET


RED.PKT EQU	*		read packet from PC
**	(HL) = buffer to store data (local or caller)
**	1st 2 bytes => incoming byte count
**	Return with next buffer address in DE

	IN	USBSTAT
	ANI	USBRX
	JZ	RED.PKT
*	JNZ	RED.PKT

	IN	USBDAT
	MOV	C,A

RED.1	EQU	*
	IN	USBSTAT
	ANI	USBRX
	JZ	RED.1
*	JNZ	RED.1

	IN	USBDAT
	MOV	B,A	(BC) = byte count 

	DCX	B
	DCX	B	subtract 2 bytes from count

IN.BLK	EQU	*	Get a block from PC
**	Enter with (BC) = byte count
**	(HL) = caller's buffer to store data
**
IN.1	IN	USBSTAT
	ANI	USBRX
	JZ	IN.1
*	JNZ	IN.1

	IN	USBDAT
	MOV	M,A
	INX	H
	DCX	B
	MOV	A,B
	ORA	C
	JNZ	IN.1

	PUSH	H
	POP	D	next address in DE
	RET

CHK.ACK EQU	*
	LDA	ACK.BUF 	1st byte of acknowledgement packet
	ORA	A		check error code returned by PC
	RZ

	STC
	RET

*  Data area . . .

B.HOLD	DB	0
C.HOLD	DB	0
LOOPCNT DB	0

CHKSUM	DB	0

ACK.BUF DS	12
	DB	0,0,0,0

SRC.DAT DS	2
SEC.NUM DS	2
DST.DAT DS	2

	DB	'PCDVD6.ASM   3/5/2010	 - 10:00 AM'
	DB	'PC260QA.ASM  02/13/2013 - 5:10 PM'

	END	ENTRY
