	TITLE	'USBTEST5.ABS'
	STL	'Dan S. Emrick, 02/13/2013'

*
*	02/13/2013: Added display of RX and TX stautus bits, line 25
*
*	02/11/2013: Changed sense of FT245 status bits 
*		  : Changed status port to base port + 2
*
*	10/16/2012: Added routines to write updated default port
*		into scratch section at the beginning of the
*		executable file using .OPENU and .POSIT

*	Send/Receive characters to validate circuit config; for
*	testing Norberto's board(s)
*	Use with simple GUI on Linux (Qt) - to be developed
*
*	Add request ability for user to specify USB-FIFO port
*	
*  Menu items:
*	1) Request port (in octal, i.e., xxxQ)
*	2) Switch to Send Character Mode
*	3) Switch to Receive and display Character Mode
*	4) Return to main menu
*	5) Exit program
*
*	No packets exchanged, only single character at a time.
*

	XTEXT	HOSDEF
	XTEXT	DOLLAR
	XTEXT	ASCII

SBC.DE	EQU	122355A
S.OMAX	EQU	040324

*USBDAT  EQU	 340Q
*USBSTAT EQU	 341Q

USBDAT	EQU	260Q
USBSTAT EQU	262Q
USBRX	EQU	2
USBTX	EQU	1

	ORG	042200A

*	Patch and default port area
SAVDPT	DB	USBDAT		default data port
SAVSPT	DB	USBSTAT 	default status port
	DB	0,0,0,0,0

START.1 EQU	*
	LDA	SAVDPT		get current/saved port
	ANI	11000000B	mask all but top two bits
	RAR
	RAR
	RAR
	RAR
	RAR
	RAR			shift to right-most two bits
	ORI	30H		add ASCII bias
	STA	DEFPT		save 1st digit

	LDA	SAVDPT
	ANI	00111000B	mask to get bits 5, 4, 3
	RAR
	RAR
	RAR			shift to right-most 3 bits
	ORI	30H		add ASCII bias
	STA	DEFPT+1 	save 2nd digit

	LDA	SAVDPT
	ANI	00000111B	mask to get bits 2, 1, 0
	ORI	30H		add ASCII bias
	STA	DEFPT+2 	save 3rd (last) digit

MENU	EQU	*
	CALL	$TYPTX
	DB	NL,NL
	DB	TAB,TAB,'Main Menu',NL,NL
	DB	TAB,'Default port set to: '
DEFPT	DB	'---Q',NL,NL
	DB	TAB,'Select function:',NL
	DB	TAB,'1	 -  Set new port number (octal)',NL
	DB	TAB,'2	 -  Go to Send Character mode',NL
	DB	TAB,'3	 -  Go to Receive & Display Character mode',NL
	DB	TAB,'4	 -  Update RXE & TXE status (line 25)',NL
	DB	TAB,'5	 -  Exit program',NL
	DB	TAB,'ESC -  Return to this menu',NL
	DB	NL,NL
	DB	'Return to main menu at any time by pressing the ESC key',NL,NL
	DB	TAB,'Enter 1, 2, 3, 4, 5, or ESC:'
	DB	' '+200Q

	LDA	SAVDPT
	STA	USBP1D		default data port for input
	STA	USBP2D		default data port for output

	LDA	SAVSPT
	STA	USBP1S		defualt status port for input
	STA	USBP2S		default status port for output
	STA	USBP3S		default status port for output

FUNC	SCALL	.SCIN
	JC	FUNC

	PUSH	PSW
	MVI	A,NL
	SCALL	.SCOUT
	POP	PSW

	CPI	'1'
	JZ	SET.PT

	CPI	'2'
	JZ	SND.CHR

	CPI	'3'
	JZ	REC.CHR

	CPI	'4'
	JZ	RTXST

	CPI	'5'
	JZ	EXIT.		restort console then exit

	CPI	ESC
	JZ	MENU

	CALL	$TYPTX
	DB	'Unknown option, please try again.'
	DB	NL,ENL
	JMP	MENU

SET.PT	EQU	*		set new default port and save to disk
	CALL	GET.OCT 	get 3 octal characters (checks keyboard)
*				returns with new port in SAVDPT
	JC	SET.PT		or some error, so try again

	MVI	A,NL
	SCALL	.SCOUT

	CALL	UPD.DEF 	save updated default to disk
	JMP	START.1 	have new port, so restart

UPD.DEF EQU	*
	MVI	A,2		channel for file info
	LXI	D,DEFALT
	LXI	H,FNAME
	SCALL	.OPENU
	JNC	READIT

	CALL	$TYPTX
	DB	'Cannot open file for update'
	DB	ENL
	RET

READIT	EQU	*
	CALL	$TYPTX
	DB	'File open for update'
	DB	ENL

	MVI	A,2
	LXI	D,BUFFER
	LXI	B,256		1st 256 bytes only
	SCALL	.READ
	JNC	CHG.WRI

	CALL	$TYPTX
	DB	'Read failed'
	DB	ENL
	RET

CHG.WRI EQU	*
	CALL	$TYPTX
	DB	'Read successful'
	DB	ENL

	MVI	A,2
	LXI	B,0
	SCALL	.POSIT		rewind to sector just read

	LDA	SAVDPT
	STA	BUFFER+8	update port number

	LDA	SAVSPT
	STA	BUFFER+9

SAV.FIL EQU	*

	MVI	A,2
	LXI	D,BUFFER
	LXI	B,256
	SCALL	.WRITE
	JNC	SAV.DON

	ORI	030H
	STA	ERR.W
	CALL	$TYPTX
	DB	'Write/Update failed: '
ERR.W	DB	'Y'
	DB	ENL

	MVI	A,2
	SCALL	.CLOSE
	XRA	A		ensure PSW cleared
	RET

SAV.DON EQU	*
	CALL	$TYPTX
	DB	'File updated'
	DB	ENL

	MVI	A,2
	SCALL	.CLOSE
	XRA	A		ensure PSW cleared
	RET

GET.OCT EQU	*
	SCALL	.SCIN		check for character
	JC	GET.OCT

	STA	PND.CHR 	save pending character
	CALL	CHK.CMD 	was this an ESC key?
	RC			carry -> ESC key

	LDA	PND.CHR 	get char back
	ANI	3		remove ASCII bias
	RAL
	RAL
	RAL
	RAL
	RAL
	RAL
	MOV	C,A		partial hex port number in C

GETOCT1 EQU	*
	SCALL	.SCIN
	JC	GETOCT1

	STA	PND.CHR
	CALL	CHK.CMD
	RC			carry -> ESC key

	LDA	PND.CHR
	ANI	7		remove ASCII bias
	RAL
	RAL
	RAL
	ORA	C		combine with pending
	MOV	C,A		save in C

GETOCT2 EQU	*
	SCALL	.SCIN
	JC	GETOCT2

	STA	PND.CHR
	CALL	CHK.CMD
	RC			carry -> ESC key

	LDA	PND.CHR
	ANI	7
	ORA	C
	STA	SAVDPT		new default port
	CPI	260Q		260Q?
	JZ	S262		yes, status port=262Q

	MVI	A,361Q		no, set status to 361Q
	JMP	S341

S262	MVI	A,262Q
S341	EQU	*
	STA	SAVSPT		new default status port
	XRA	A		be sure carry clear
	RET			

SND.CHR EQU	*		get char from keyboard and send to PC
	SCALL	.SCIN
	JC	SND.CHR 	wait for keyboard

	STA	PND.CHR 	save pending character
	CALL	CHK.CMD 	check for  ESC key
	JC	MENU		ESC key hit, restart (return to main)

IN.STAT EQU	*
	IN	USBSTAT 	not ESC, so see if USB is ready
USBP2S	EQU	*-1		patch location for status port chagnge
	ANI	USBTX		check for empty FIFO on USB card
	JZ	SND.CHR

	LDA	PND.CHR 	get char back and send it
	OUT	USBDAT
USBP2D	EQU	*-1		patch location for port change
	JMP	SND.CHR 	repeat until ESC detected

REC.CHR EQU	*		wait for char from PC and display on screen
	IN	USBSTAT
USBP1S	EQU	*-1		patch location for status port chagnge
	ANI	USBRX
	JNZ	IN.CHR

	CALL	CHK.KBD 	no char yet, so check for ESC from keyboard 
	JC	MENU
	JMP	REC.CHR 	no char and no ESC, check USB input again

IN.CHR	EQU	*
	IN	USBDAT		data waiting, get it
USBP1D	EQU	*-1		patch location for port change
	SCALL	.SCOUT		show user
	MVI	A,NL
	SCALL	.SCOUT		with new line
	JMP	REC.CHR 	loop until ESC from keyboard

CHK.KBD EQU	*
	SCALL	.SCIN

CHK.CMD CPI	ESC
	RNZ			not ESC key, so return w/carry clear

	STC			got ESC key, set carry and return
	RET

*	Come here to restore console and exit

CC.XIT	EQU	*
	CALL	$TYPTX
	DB	NL,'Control-C exit or abort'
	DB	ENL

EXIT.	EQU	*
	MVI	B,80
	MVI	C,-1
	MVI	A,3
	SCALL	.CONSL		restore 80 char screen width
	MVI	B,203Q		BS, no case map, map BKSP, term. proc. tab
	MVI	C,-1
	MVI	A,1
	SCALL	.CONSL

	XRA	A
	SCALL	.EXIT

RTXST	EQU	*		show on Line 25
	IN	USBSTAT
USBP3S	EQU	*-1		patch location for status port chagnge
	STA	STAVAL		store for RX test and hex display conversion
	ANI	USBTX		mask for TX status
	JNZ	OK.TX

	LXI	D,TXBLK
	LXI	H,TXTGT
	LXI	B,21
	CALL	$MOVE
	JMP	RTXST.1

OK.TX	EQU	*
	LXI	D,TXGO
	LXI	H,TXTGT
	LXI	B,21
	CALL	$MOVE

RTXST.1 EQU	*
	LDA	STAVAL
	ANI	USBRX
	JNZ	OK.RX

	LXI	D,RXBLK
	LXI	H,RXTGT
	LXI	B,21
	CALL	$MOVE
	JMP	UPD25

OK.RX	EQU	*
	LXI	D,RXGO
	LXI	H,RXTGT
	LXI	B,21
	CALL	$MOVE

UPD25	EQU	*		show FT245 status bits on line 25
	LDA	STAVAL		get raw status value
	LXI	H,HEXD		point to destination
	CALL	SHO.HEX

	CALL	$TYPTX
	DB	ESC,'j' 	save cursor postion
	DB	ESC,'p' 	reverse video on
	DB	ESC,'x1',ESC,'Y8 ',TAB
	DB	'TX: '
TXTGT	DB	'---------------------',TAB
	DB	'RX: '
RXTGT	DB	'---------------------',TAB
HEXD	DB	'  H'
	DB	ESC,'q' 	reverse video off
	DB	ESC,'k' 	restore cursor position
	DB	200Q
	XRA	A
	JMP	MENU

SHO.HEX EQU	*		convert val in A to 2-digit hex
*				and place in memory
ASC.2	PUSH	PSW
	RRC
	RRC
	RRC
	RRC
	CALL	ASC.3
	POP	PSW
ASC.3	ANI	0FH
	ADI	'0'
	CPI	':'
	JC	ASC.4
	ADI	7
ASC.4	MOV	M,A
	INX	H
	RET

RXBLK	DB	'Do not read from FTDI',0
RXGO	DB	'OK  to read from FTDI',0

TXBLK	DB	'Do not write to FTDI ',0
TXGO	DB	'OK  to write to FTDI ',0

* Data area followed by initialization code that is later overwritten

STAVAL	DB	0

OLDRX	DB	0
OLDTX	DB	0

PND.CHR DB	0
DEFALT	DB	'SY0ABS'
FNAME	DB	'SY0:USBTEST6.ABS',0

BUFFER	EQU	*

* Initialization code; overwritten later

START	EQU	*		set up control-C exit
	MVI	A,-1
	SCALL	.CLOSE		close chan we came in on

	MVI	A,3
	LXI	H,CC.XIT
	SCALL	.CTLC

	MVI	A,3
	MVI	C,0
	SCALL	.CONSL

	MVI	A,3
	MVI	B,-1
	MVI	C,-1
	SCALL	.CONSL

	MVI	A,1
	MVI	C,0
	SCALL	.CONSL

	MVI	A,0
	MVI	B,1
	MVI	C,1
	SCALL	.CONSL		set character mode

	MVI	A,0		load both overlays
	SCALL	.LOADO
	JC	OVL.ERR

	MVI	A,1
	SCALL	.LOADO
	JC	OVL.ERR

	LXI	H,-1
	SCALL	.SETTOP
	LXI	D,S.OMAX
	DW	SBC.DE
	LXI	D,-10
	DAD	D
	SHLD	MAXMEM
	SCALL	.SETTOP

	LHLD	MAXMEM		show user top of memory
	MVI	A,6
	MOV	B,H
	MOV	C,L
	LXI	H,HIMEM
	CALL	$UDD

	JMP	START.1


	CALL	$TYPTX
	DB	12Q
	DB	'High memory limit established - '
HIMEM	DB	'000000',ENL

	JMP	START.1

SET.ERR EQU	*
	CALL	$TYPTX
	DB	'.SETTOP error'
	DB	ENL
	JMP	CC.XIT

OVL.ERR EQU	*		not needed unless/until disk access added
	CALL	$TYPTX
	DB	'.LOADO error'
	DB	ENL
	JMP	CC.XIT

MAXMEM	DB	0,0
TAG.0	DB	'USBTEST1.ASM,	10/16/2012,   7:45 AM',0
	DB	'USBTEST2.ASM,	10/16/2012,   9:00 AM',0
	DB	'USBTEST3.ASM,	02/11/2013,   12:05 PM',0
	DB	'USBTEST4.ASM,	02/13/2013,   10:50 AM',0
	DB	'USBTEST4.ASM,	02/16/2013,    8:30 AM',0
	DB	'USBTEST4.ASM,	02/16/2013,    4:15 PM',0
	DB	'USBTEST4.ASM,	02/17/2013,    7:00 PM',0
	DB	'USBTEST5.ASM,	02/18/2013,    8:40 AM',0
	DB	'USBTEST6.ASM,	03/04/2013,   10:15 AM',0
	DB	0,0,0,0,0,0,0,0

	END	START
